package com.chelvc.cloud.maintain.controller;

import java.util.Map;
import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;

import com.chelvc.cloud.vehicle.client.OmsOrderClient;
import com.chelvc.cloud.vehicle.client.OmsOrderReturnApplyClient;
import com.chelvc.cloud.vehicle.client.dto.ConfirmOrderResultDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderDetailDTO;
import com.chelvc.cloud.vehicle.client.dto.OmsOrderReturnApplyDTO;
import com.chelvc.cloud.vehicle.client.param.OmsOrderModifyParam;
import com.chelvc.cloud.vehicle.client.param.OrderPagingParam;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.common.model.Pagination;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * 订单接口
 *
 * @author liude
 * @Date 2023/11/19
 **/
@Validated
@RestController
@ResponseWrapping
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderController {
    private final OmsOrderClient omsOrderClient;
    private final OmsOrderReturnApplyClient omsOrderReturnApplyClient;

    /**
     * 根据购物车信息生成确认单信息
     *
     * @param orderParam
     * @return
     */
    @PostMapping("/generateConfirmOrder")
    public ConfirmOrderResultDTO generateConfirmOrder(@RequestBody @Valid OmsOrderModifyParam orderParam) {
        return this.omsOrderClient.generateConfirmOrder(orderParam);
    }

    /**
     * 根据购物车信息生成订单
     *
     * @param orderParam
     * @return
     */
    @PostMapping("/generateOrder")
    public Map<String, Object> generateOrder(@RequestBody @Valid OmsOrderModifyParam orderParam) {
        Map<String, Object> result = omsOrderClient.generateOrder(orderParam);
        return result;
    }

    /**
     * 查询订单分页
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/order/paging")
    public Pagination<OmsOrderDTO> getOrderPaging(@Valid OrderPagingParam param) {
        return this.omsOrderClient.getOrderPaging(param);
    }

    /**
     * 确认订单
     *
     * @param id 查询参数
     */
    @GetMapping("/confirmReceiveOrder/{id}")
    public void confirmReceiveOrder(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id) {
        this.omsOrderClient.confirmReceiveOrder(id);
    }

    /**
     * 手动取消订单
     *
     * @param id 查询参数
     */
    @GetMapping("/cancelOrder/{id}")
    public void cancelOrder(@PathVariable("id") @Min(value = 1, message = "订单ID不能小于1") Long id) {
        this.omsOrderClient.cancelOrder(id);
    }


    /**
     * 删除订单
     *
     * @param id 查询参数
     */
    @GetMapping("/deleteOrder/{id}")
    public void deleteOrder(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id) {
        this.omsOrderClient.deleteOrder(id);
    }

    /**
     * 支付成功回调
     *
     * @param id 查询参数
     */
    @GetMapping("/paySuccess/{id}")
    public void paySuccess(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id,
                           Integer payType) {
        this.omsOrderClient.paySuccess(id, payType);
    }

    /**
     * 查看退款订单信息
     *
     * @param id
     * @return
     */
    @GetMapping("/getOrderReturnApplyInfo")
    public OmsOrderReturnApplyDTO getOrderReturnApplyInfo(@PathVariable("id") Long id) {
        return this.omsOrderReturnApplyClient.getOrderReturnApplyInfo(id);
    }


    /**
     * 商家端查询订单分页
     *
     * @param param 查询参数
     * @return 订单分页信息
     */
    @GetMapping("/order/merchantPaging")
    public Pagination<OmsOrderDTO> getMerchantOrderPaging(@Valid OrderPagingParam param) {
        return this.omsOrderClient.getMerchantOrderPaging(param);
    }


    /**
     * 商家端查询订单量以及今日收入
     *
     * @return 订单量以及今日收入
     */
    @GetMapping("/order/getOrderQuantity/{merchantId}")
    public Map<String, Object> getOrderQuantity(
            @PathVariable("merchantId") @Min(value = 1, message = "商家ID不能小于1") Long merchantId) {
        return this.omsOrderClient.getOrderQuantity(merchantId);
    }

    /**
     * 订单核验
     *
     * @param orderId
     * @param merchantId
     * @return
     */
    @GetMapping("/order/verification/{orderId}/{merchantId}")
    public void orderVerification(@PathVariable("orderId") @NotNull(message = "订单错误") Long orderId,
                                  @PathVariable("merchantId") @NotNull(message = "订单错误") Long merchantId) {
        this.omsOrderClient.orderVerification(orderId, merchantId);
    }

    /**
     * 查询订单详情
     *
     * @param id 查询参数
     * @return
     */
    @GetMapping("/order/{id}")
    public OmsOrderDetailDTO getOrderDetail(@PathVariable("id") @Min(value = 1, message = "主键不能小于1") Long id) {
        return this.omsOrderClient.detail(id);
    }
}
